@@ -1,3 +1,3 @@
-from rl_helper.envhelper import envhelper
+from rl_helper.envhelper import envhelper,VDisplay
 from rl_helper.fps import fps
 from rl_helper.exps import ExperimentManager
\ No newline at end of file
